<?php
/**
 * Created by kuriernet.pl
 */

class KurierNetApi2 extends KurierNetAPI
{
    /**
     * @param array $pobierzGodzinyNadania
     * @return bool|object
     *      {
                [DataNadania] date
                [GodzinaNadaniaOd] string
                [GodzinaNadaniaDo] string
                [NajpozniejszaGodzinaNadania] string
                [Odstep] => int
     *      }
     */
    public function pobierzGodzinyNadania(array $pobierzGodzinyNadania)
    {
        $result = $this->callAPI('pobierzGodzinyNadania', $pobierzGodzinyNadania);
        if ($result!==false){
            return $result->GodzinyNadania;
        }
        return $result;
    }

    /**
     * @param array $obliczCenePrzesylki
     * @return bool|object
     *      {
                [Cena] decimal
                [Opis] html string
     *      }
     */
    public function obliczCenePrzesylki(array $obliczCenePrzesylki)
    {
        $result = $this->callAPI('obliczCenePrzesylki', $obliczCenePrzesylki);
        if ($result!==false){
            return $result->CenaResponse;
        }
        return $result;
    }

    /**
     * @param array $dodajPrzesylki
     * @return bool|object
     *      {
                [NrPrzesylki] string
                [NrNadania] string
                [NrReferencyjny] numeric string
                [Cena] decimal
     *      }
     */
    public function dodajPrzesylki(array $dodajPrzesylki)
    {
        $result = $this->callAPI('dodajPrzesylki', $dodajPrzesylki);
        if ($result!==false){
            return $result->PrzesylkaResponse;
        }
        return $result;
    }

    /**
     * @param array $pobierzEtykiety
     * @return bool|object
     *      {
                [NrPrzesylki] string
                [Etykieta] hex string
     *      }
     */
    public function pobierzEtykiety(array $pobierzEtykiety)
    {
        $result = $this->callAPI('pobierzEtykiety', $pobierzEtykiety);
        if ($result!==false){
            return $result->Etykiety;
        }
        return $result;
    }
}

class KurierNetAPI
{
    public static $WSDL='https://www.kuriernet.pl/shipping/?wsdl';

    protected $apiLogin;
    protected $apiKey;
    protected $client; //SoapClient
    protected $logFile; //File to log
    protected $methodName; //call method name
    protected $paramList; //list of parameters for call method

    protected $objRequest=null;  //$objRequest to SOAP
    protected $objResponse=null; //$bojResponse from SOAP
    protected $objFault=null; //case $objFault from SOAP
    protected $objException=null; //Trawobject with full exception data
    private $errorList=null;//case fault array of [Kod=>Komunikat]
    private $faultcode;  //msg with fault type
    private $faultstring; //msg with fault description

    public function getClient(){
        return $this->client;
    }//SoapClient
    public function getLogFile(){
        return $this->logFile;
    }//File to log
    public function getMethodName(){
        return $this->methodName;
    }//call method name
    public function getParamList(){
        return $this->paramList;
    }//list of parameters for call method

    public function getObjRequest(){
        if (empty($this->objRequest)){
            return false;
        }
        return $this->objRequest;
    } //$objRequest to SOAP
    public function getObjResponse(){
        if (empty($this->objResponse)){
            return false;
        }
        return $this->objResponse;
    }//$bojResponse from SOAP
    public function getObjFault(){
        if (empty($this->objFault)){
            return false;
        }
        return $this->objFault;
    }//case $objFault from SOAP
    public function getObjException(){
        if (empty($this->objException)){
            return false;
        }
        return $this->objException;
    }//Trawobject with full exception data
    public function getErrorList(){
        if (empty($this->errorList)){
            return false;
        }
        return $this->errorList;
    }//case fault array of [Kod=>Komunikat]
    public function getFaultCode(){
        return $this->faultcode;
    } //msg with fault type
    public function getFaultString(){
        return $this->faultstring;
    }//msg with fault description


    public function __construct($WSDL='')
    {
        if (empty($WSDL)){
            $WSDL=self::$WSDL;
        }
        $this->client = $this->initSoapClient($WSDL);
    }

    /**
     * Set file to writing log
     * @param $logFile
     */
    public function setLogFile($logFile){
        $this->logFile=$logFile;
    }//File to log

    /**
     * Set authorization paramitoers to call each API method
     * @param $apiLogin --authorization login
     * @param $apiKey   --authorization unique key
     */
    public function setAuthorization($apiLogin,$apiKey){
        $this->apiLogin=$apiLogin;
        $this->apiKey=$apiKey;
    }

    /**
     * Call API kuriernet.pl method with array parameters
     * @param $methodName
     * @param $paramList --array parameters
     * @return bool|object| --false = fault case
     */
    public function callAPI($methodName,$paramList)
    {
        $this->paramList=$paramList;
        if (!is_array($paramList)){
           $this->paramList=array($paramList);
        }
        if (isset($this->apiLogin)){
            $this->paramList['Autoryzacja']['Login'] = $this->apiLogin;
            $this->paramList['Autoryzacja']['KluczAPI'] = $this->apiKey;
        }
        $this->methodName=$methodName;
        $this->writeLog("__soapCall: " . $this->methodName . json_encode($this->paramList), $this->logFile);
        $result = $this->callMethod($this->client, $this->methodName, $this->paramList);
        if ($result===false){
            $this->errorLog();
        }
        return $result;

    }

    protected function initSoapClient($WSDL)
    {
        $mode = array('trace' => true);
        $client = new SoapClient($WSDL, $mode);
        return $client;
    }

    protected function writeLog($message, $logFile)
    {
        //saving the log depends on define log file name
        if (!empty($logFile)) {
            file_put_contents($logFile, $message, FILE_APPEND);
        }
    }

    protected function errorLog()
    {
        $this->writeLog( $this->faultcode . ': ' . $this->faultstring, $this->logFile);
        //$this->writeLog("__getLastRequest: ".$this->client->__getLastRequest(),$logFile);
        $this->writeLog("__getLastResponse: " . $this->client->__getLastResponse(), $this->logFile);
    }

    private function extractErrorList($faultResult)
    {
        $result=array();
        if (isset($faultResult->Bledy) && isset($faultResult->Bledy->Blad)) {
            //write list of error info message to errorList
            $err = $faultResult->Bledy->Blad;
            if (count($err) > 1) {
                foreach ($err as $e) {
                    $result[$e->Kod]=$e->Komunikat;
                }
            } else {
                $result[$err->Kod]=$err->Komunikat;
            }
        }
        return $result;
    }

    private function callMethod(SoapClient $client, $methodName, $params)
    {
        $result=false;
        try {
            $this->objResponse = $client->__soapCall($methodName, array($params));
            $result = $this->objResponse;
        }catch(Exception $ex){
            $this->objException = $ex;
            $this->faultcode = $ex->faultcode;
            $this->faultstring = $ex->faultstring;
            //Extract errors from detail fault info
            //If is there Fault object structure
            if (isset($ex->detail)) {
                $fault = get_object_vars($ex->detail);
                $faultResult = reset($fault);
                $this->objFault = $faultResult;
                $this->errorList = $this->extractErrorList($faultResult);
            } else {
                //
            }
        }
        return $result;
    }
}
